unit AbortBox; {use with UPRINT}

interface

uses
  Windows,
  Messages, SysUtils, Classes, Controls, Forms, Buttons, StdCtrls, ExtCtrls,
  Gauges, EZ_Print;

type

  { TAbortForm }
  TAbortForm = class(TForm)
    AbortBtn: TBitBtn;
    Panel1: TPanel;
    Bevel1: TBevel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Bevel2: TBevel;
    Gauge1: TGauge;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure AbortBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure AbortBtnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FObjects: Longint;
    JP: TEZ_Printer;
    procedure DoAbortPrinting(UserAborted: Boolean);
    procedure WMCommand(var Msg: TMessage); message wm_Command;
    procedure WMGetMinMaxInfo(var Msg: TMessage); message WM_GETMINMAXINFO;
    procedure WMNCHitTest(var Msg: TMessage); message WM_NCHITTEST;
    procedure SetObjects(Value: Longint);
    procedure SetProgress(Value: Longint);
  public
    Locks: Integer;
    CurPage, NrPages: Integer;
    constructor Create(AOwner: TComponent; AEZ_Print: TEZ_Printer);
    procedure SetLabels(ATitle: String; AMaxObjects: Integer);
    procedure UpdatePages(No: Integer);
    procedure EnableShow;
    property NoObjects: Longint read FObjects write SetObjects default 0;
    property Progress: Longint write SetProgress;
  end;

const
  AbortForm: TAbortForm = nil;

procedure CreateAbortForm(AOwner: TComponent; ATitle: String; EZ: TEZ_Printer);
procedure DestroyAbortForm;

implementation

{$R *.DFM}

uses
  EQ_Abort;

procedure CreateAbortForm(AOwner: TComponent; ATitle: String; EZ: TEZ_Printer);
begin
  if not Assigned(AbortForm) then
  begin
    if not Assigned(AOwner) then
      AOwner := Screen.ActiveForm;
    nopActiveWindow:= TWinControl(AOwner).Handle;
    nopWindowList := DisableTaskWindows(0);
    AbortForm := TAbortForm.Create(AOwner, EZ);
  end;
  AbortForm.SetLabels(ATitle, EZ.PageHeight);
end;

procedure DestroyAbortForm;
begin
  if Assigned(AbortForm) then
  begin
    Dec(AbortForm.Locks);
    if AbortForm.Locks > 0 then Exit;
    EnableTaskWindows(nopWindowList);
    SetActiveWindow(nopActiveWindow);
    AbortForm.Free;
    AbortForm := nil;
  end;
end;

{ TAbortDialog }
constructor TAbortForm.Create(AOwner: TComponent; AEZ_Print: TEZ_Printer);
begin
  inherited Create(AOwner);
  JP := AEZ_Print;
end;

procedure TAbortForm.FormCreate(Sender: TObject);
begin
  EnableMenuItem(GetSystemMenu(Handle, False), sc_Close, mf_Grayed);
  Locks := 0;
end;

procedure TAbortForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := not JP.Printing or UserAbort;
end;

procedure TAbortForm.SetLabels(ATitle: String; AMaxObjects: Integer);
begin
  Label2.Caption := ATitle;
  Label3.Caption := Format('on the %s', [JP.CurDevice]);
  if pos('Printer', Label3.Caption) = 0 then
    Label3.Caption := Label3.Caption + ' Printer';
  Label4.Caption := Format('connected to %s', [JP.CurPort]);
  Label5.Caption := '';
  Inc(Locks);
  CurPage := 0;
  NrPages := 0;
end;

procedure TAbortForm.SetObjects(Value: Longint);
begin
  if FObjects <> Value then
  begin
    FObjects := Value;
    Gauge1.MaxValue := Value;
  end;
end;

procedure TAbortForm.SetProgress(Value: Longint);
begin
  Gauge1.Progress := Value;
end;

procedure TAbortForm.UpdatePages(No: Integer);
begin
  if No = 0 then DoAbortPrinting(True)
  else
  begin
    if No < 0 then
    begin
      if No = -9999 then NrPages := 9999
      else NrPages := -No;
      if not JP.Collate then
        NrPages := NrPages * JP.Copies;
    end;

    if No = 1000 then CurPage := NrPages;
    if CurPage >= NrPages then
    begin
      if CurPage > NrPages then Dec(CurPage);
      Label5.Caption := 'End of Printing. (please wait...)';
    end
    else
    begin
      if JP.Collate then Inc(CurPage)
      else Inc(CurPage, JP.Copies);
      if NrPages < 9999 then
        if NrPages = 1 then Label5.Caption := 'Printing document'
        else Label5.Caption := Format('Page %d of %d', [CurPage, NrPages])
      else Label5.Caption := Format('Page %d', [CurPage]);
    end;
    Label5.Update;
  end;
end;

procedure TAbortForm.DoAbortPrinting(UserAborted: Boolean);
begin
  if UserAborted and JP.Printing then
  begin
    Label5.Caption := 'Abort printing, wait...';
    Label5.Refresh;
    UserAbort := True;
    JP.Abort;
  end;
end;

procedure TAbortForm.WMGetMinMaxInfo(var Msg: TMessage);
type
  PPointArray = ^TPointArray;
  TPointArray = array [0..4] of TPoint;
begin
  inherited;
  PPointArray(Msg.LParam)^[3].X := 273;
  PPointArray(Msg.LParam)^[3].Y := 191;
  PPointArray(Msg.LParam)^[4] := PPointArray(Msg.LParam)^[3];
end;

procedure TAbortForm.WMNCHitTest(var Msg: TMessage);
begin
  inherited;
  if (Msg.Result in [HTLEFT..HTBOTTOMRIGHT]) then
    Msg.Result := 0;
end;

procedure TAbortForm.WMCommand(var Msg: TMessage);
begin
  DoAbortPrinting(True);
end;

procedure TAbortForm.AbortBtnClick(Sender: TObject);
begin
  DoAbortPrinting(True);
end;

procedure TAbortForm.AbortBtnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  DoAbortPrinting(Key in [VK_RETURN, VK_ESCAPE]);
end;

procedure TAbortForm.FormShow(Sender: TObject);
var
  X, Y: Integer;
begin
  X := (Screen.Width - Width) div 2;
  Y := (Screen.Height - Height) div 2;
  if Application.MainForm <> nil then
  begin
    if not Application.MainForm.Visible then Exit;
    X := Application.MainForm.Left + (Application.MainForm.Width  - Width)  div 2;
    Y := Application.MainForm.Top  + (Application.MainForm.Height - Height) div 2;
  end;
  SetBounds(X, Y, Width, Height);
end;

procedure TAbortForm.EnableShow;
begin
  Timer1.Enabled := True;
end;

procedure TAbortForm.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled := False;
  Show;
  Invalidate;
  Update;
end;

end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

