{*******************************************************}
{                                                       }
{      API Tools                                        }
{      ---------                                        }
{      ADV Compiled                                     }
{      Copyright (C), Lucian Radulescu                  }
{      E-mail:    ljee@interlog.com                     }
{      Home Page: http://www.interlog.com/~ljee         }
{                                                       }
{*******************************************************}

unit ApiTools;

interface

uses
  Windows,
  Classes, Graphics, Forms, ExtCtrls;

function GetLeftButton: Byte;
{return the mapped left button}

procedure FillRectSolid(DC: HDC; const R: TRect; Color: TColor);
{Paint the given rectangle with Color}

procedure SwapForms(NewForm, OldForm: THandle; SW_WHAT: Word);

procedure DC_DrawTransparentBitmap(DC: HDC; Bmp: HBITMAP; xStart, yStart: Integer;
  cTransparentColor: LongInt);

procedure Make_BMP_Transparent(I: TImage; BckgColor: TColor);

implementation

var
  OldAppFocus: HWnd;     { used by SwapForms to save and reset focus       }
  WindowsList: Pointer;  { used by SwapForms to enable and disable windows }

procedure SwapForms(NewForm, OldForm: THandle; SW_WHAT: Word);
var
  R: TRect;
begin
  GetWindowRect(OldForm, R);
  with R do
    MoveWindow(NewForm, Left, Top, Right-Left, Bottom-Top, True);
  if SW_WHAT = SW_HIDE then
  begin
    OldAppFocus := GetActiveWindow;
    WindowsList := DisableTaskWindows(0);
    ShowWindow(NewForm, SW_SHOW);
    ShowWindow(OldForm, SW_WHAT);
  end
  else
  begin
    EnableTaskWindows(WindowsList);
    ShowWindow(NewForm, SW_SHOW);
    SetActiveWindow(OldAppFocus);
  end;
end;

function GetLeftButton: Byte;
const
  RLButton:array[Boolean] of Word = (VK_LBUTTON, VK_RBUTTON);
begin
  Result := RLButton[GetSystemMetrics(SM_SWAPBUTTON) <> 0];
end;

procedure FillRectSolid(DC: HDC; const R: TRect; Color: TColor);
var
  B, OldB: HBrush;
begin
  B := CreateSolidBrush(ColorToRGB(Color));
  OldB := SelectObject(DC, B);
  try
    if GetDeviceCaps(DC, RASTERCAPS) AND RC_BITBLT = RC_BITBLT then
      PatBlt(DC, R.Left, R.Top, R.Right-R.Left, R.Bottom-R.Top, PATCOPY)
    else FillRect(DC, R, B);
  finally
    DeleteObject(SelectObject(DC, OldB));
  end;
end;

procedure DC_DrawTransparentBitmap(DC: HDC; Bmp: HBITMAP; xStart, yStart: Integer;
  cTransparentColor: LongInt);
var
  bm: Windows.TBitmap;
  ptSize: Windows.TPoint;
  cColor: TColorRef;
  bmAndBack, bmAndObject, bmAndMem, bmSave, oldBmp: HBITMAP;
  bmBackOld, bmObjectOld, bmMemOld, bmSaveOld, hBmp: HBITMAP;
  hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave: HDC;
  temp_bitmap: TBitmap;
begin
  temp_bitmap := TBitmap.Create;
  temp_bitmap.Handle := Bmp;
  try
    hBmp := temp_bitmap.Handle;
    hdcTemp := CreateCompatibleDC(dc);
    oldBmp := SelectObject(hdcTemp, hBmp);

    GetObject(hBmp, SizeOf(bm), @bm);
    ptSize.x := bm.bmWidth;
    ptSize.y := bm.bmHeight;

    hdcBack   := CreateCompatibleDC(dc);
    hdcObject := CreateCompatibleDC(dc);
    hdcMem    := CreateCompatibleDC(dc);
    hdcSave   := CreateCompatibleDC(dc);

    bmAndBack   := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);
    bmAndObject := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);
    bmAndMem    := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);
    bmSave      := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);

    bmBackOld   := SelectObject(hdcBack, bmAndBack);
    bmObjectOld := SelectObject(hdcObject, bmAndObject);
    bmMemOld    := SelectObject(hdcMem, bmAndMem);
    bmSaveOld   := SelectObject(hdcSave, bmSave);

    SetMapMode(hdcTemp, GetMapMode(dc));

    BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);
    cColor := SetBkColor(hdcTemp, cTransparentColor);
    BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);
    SetBkColor(hdcTemp, cColor);

    BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, dc, xStart, yStart, SRCCOPY);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
    BitBlt(dc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0, SRCCOPY);
    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

    DeleteObject(SelectObject(hdcBack, bmBackOld));
    DeleteObject(SelectObject(hdcObject, bmObjectOld));
    DeleteObject(SelectObject(hdcMem, bmMemOld));
    DeleteObject(SelectObject(hdcSave, bmSaveOld));

    SelectObject(hdcTemp, oldBmp);

    DeleteDC(hdcMem);
    DeleteDC(hdcBack);
    DeleteDC(hdcObject);
    DeleteDC(hdcSave);
    DeleteDC(hdcTemp);
  finally
    temp_bitmap.ReleaseHandle;
    temp_bitmap.Free;
  end;
end;

procedure Make_BMP_Transparent(I: TImage; BckgColor: TColor);
var
  R: TRect;
  TempBMP: TBitmap;
begin
  TempBMP := TBitMap.Create;
  try
    TempBMP.Assign(I.Picture.BitMap);
    with TempBMP do R := Rect(0, 0, Width, Height);
    I.Picture.BitMap.Canvas.Brush.Color := clBtnFace;
    I.Picture.BitMap.Canvas.BrushCopy(R, TempBMP, R, BckgColor);
  finally
    TempBMP.Free;
  end;
end;

end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

