unit DCTools;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  Classes, SysUtils;

type

  {$IFNDEF ATLEAST_D4}
  LongWord = Longint;
  {$ENDIF ATLEAST_D4}

  { TRect_Struct is used by DRAWPATTERNRECT based methods }
  TRect_Struct = packed record
    ptPosition : TPoint;
    ptSize     : TPoint;
    wStyle     : Word;
    wPattern   : Word;
  end;


procedure EscapeDRAWPattern(ADC: HDC; RS: TRect_Struct);
function  GetAveWidth(DC: HDC): Integer;
procedure SureGetTextMetric(DC: HDC; var TextMetric: TTextMetric);

implementation

uses
  CheckGDI;

procedure SureGetTextMetric(DC: HDC; var TextMetric: TTextMetric);
var
  TM: Pointer;
  cbSize: Word;
begin
  cbSize := GDICheck(GetOutlineTextMetrics(DC, 0, nil));
  GetMem(TM, cbSize);
  try
    FillChar(TM^, cbSize, #0);
    TOutlineTextMetric(TM^).otmSize := cbSize;
    GDICheck(GetOutlineTextMetrics(DC, cbSize, TM));
    Move(TOutlineTextMetric(TM^).otmTextMetrics, TextMetric, SizeOf(TextMetric));
  finally
    FreeMem(TM, cbSize);
  end;
end;

function GetAveWidth(DC: HDC): Integer;
var
  TM: TTextMetric;
begin
  SureGetTextMetric(DC, TM);
  Result := TM.tmAveCharWidth;
end;

procedure EscapeDRAWPattern(ADC: HDC; RS: TRect_Struct);
var
  Res: {$IFDEF WIN32}LongWord{$ELSE}Integer{$ENDIF WIN32};
begin
  Res := {$IFDEF WIN32}ExtEscape{$ELSE}Escape{$ENDIF WIN32}
    (ADC, DRAWPATTERNRECT, SizeOf(TRect_Struct), @RS, {$IFDEF WIN32}0, {$ENDIF}nil);
  if Res <= 0 then { check the comments below, for removing 0 here }
    if Res = 0 then
	{raise EGDILIB.Create('GDI: Escape "DRAWPATTERNRECT" not implemented.')
	 Oh boy, nobody wants to see this HPIV's error
	 never show the above error, cuz some buggy printer drivers might return
	 0 (zero) even though it didn't failed (e.g.: HP IV's), case in which
	 they should return 1. Check Microsoft's API help }
    else
    if Res = LongWord(-1) then  // SP_ERROR
      raise EGDI.Create('GDI problem: General error.')
    else
    if Res =  LongWord(-4) then // SP_OUTOFDISK
      raise EGDI.Create('GDI problem: Not enough disk space available for spooling.')
    else
    if Res = LongWord(-5)then   // SP_OUTOFMEMORY
      raise EGDI.Create('GDI problem: Not enough memory available for spooling.')
    else
      raise EGDI.Create('GDI problem: Unknown error.');
end;

end.

