{*******************************************************}
{                                                       }
{       EZ_Print 4.0 unit - Print procedures            }
{       ------------------------------------            }
{       ADV32 Compiled & Tested                         }
{       under Windows 9x, NT4, W2K                      }
{       Copyright  1997-2000, Lucian Radulescu         }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit CustomPP;

interface

uses
  Windows,
  SysUtils, Classes, Graphics, StdCtrls,
  PCharLIB, PrintPro;

type

  { TCustomProcs }
  TCustomProcs = class(TPrinterProcs)
  protected
    function GetFontHeightNoInternalLeading: Integer; {uses current font}
    procedure SetFont(Sender: TObject;
		      var FontSize: Integer;
		      var FontName: TFontName;
		      var FontStyle: TJeeFontStyles);
  public
    constructor Create(AOwner: TComponent); override;
    procedure PrintCOARM;
    procedure PrintExecutions(Y1, Y2: Integer; HasExecutions, CheckProp, CheckExec: Boolean);
    procedure Print_123(Y1: Integer; CheckReg, CheckLand, CheckProp: Boolean;
                        NoPages: Integer; Memo: TCustomMemo);
    procedure SeeScheduleCheck(X, Y: Integer; Checked: Boolean);
    procedure PrintConsideration(Y1: Integer; const Prompt: String; RoundTopRight: Boolean);
    procedure PrintScaledMemoText(X, Y, W, CheckX, CheckY, InterLines: Integer;
                                  Memo: TCustomMemo; Squeeze, FlushIt: Boolean);
    procedure PrintPropID(X, Y, W, InterLines: Integer; Memo: TCustomMemo);
    procedure PrintAmount(const Prompt: String; Y, H: Integer; HasRect: Boolean);
    procedure PrintDescription(const Prompt: String; Y1, Y2, Y3: Integer; Memo: TCustomMemo);
    procedure PrintFeesAndTax(Y1, Y2: Integer; S1, S2: String);
    procedure PrintEndorsement(X, Y: Integer; const Endorsement: String);
    procedure DocContains(X, Y: Integer; IsAorB, chkSketch, chkDescr, chkParties, chkOther: Boolean);
    procedure PrintAssessment(IsA: Boolean; Y1, Y2: Integer);
    procedure PrintMunAddProp(X1, Y1, X2, Y2: Integer; Prompt1, Prompt2: String);
    procedure PrintDocPrepBy(X1, Y1, X2, Y2: Integer; Prompt: String);
    procedure PrintBox7Fees(Prompt: String);
    procedure YMD_DateAndPrompts(Prompt1, Prompt2: String; X, Y1, Y2: Integer);
  end;

const
  LangUsed: Integer = 0;

function GetEndorsement(IsPolaris: Boolean): String;

implementation

uses
  CheckGDI, DCTools;

function GetEndorsement(IsPolaris: Boolean): String;
begin
  Result := ''; //'EZ Print 4.0';
end;

constructor TCustomProcs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnSetFont := SetFont;
end;

procedure TCustomProcs.SetFont;
begin
  if not UsePointSize then
    FontSize := Trunc(4.0 / 3.0 * FontSize);
  if FontName = '' then FontName := 'Arial'
  else if FontName = 'TNR' then FontName := 'Times New Roman';
end;

function TCustomProcs.GetFontHeightNoInternalLeading: Integer;
var
  TM: TTextMetric;
begin
  SureGetTextMetric(DC, TM);
  with TM do Result := tmHeight-tmInternalLeading;
end;

procedure TCustomProcs.PrintCOARM;

  procedure PrintIt(X: Integer; S1, S2, S3: String);
  begin
    SpoolText(X,   0, 190, 0, 0, S1, '', [], tajLeftJustify);
    SpoolText(X, 250, 190, 0, 0, S2, '', [], tajLeftJustify);
    SpoolText(X, 500, 190, 0, 0, S3, '', [], tajLeftJustify);
  end;

var
  X: Integer;
begin
  GDICheck(SetBkMode(DC, TRANSPARENT));
  if LangUsed = 0 then X := 3400
  else X := 2700;
  PrintBMPs(X-1000, 0, X-200, 800, 'COARM', TRANSPARENT);
  PrintIt(X, 'Province', 'of', 'Ontario');
  if LangUsed = 1 then PrintIt(X+1100, 'Province', 'de', 'l''Ontario');
end;

procedure TCustomProcs.SeeScheduleCheck(X, Y: Integer; Checked: Boolean);
begin
  SpoolText(X,     Y, 190, 0, 0, 'Additional:', '', [], tajLeftJustify);
  SpoolText(X, Y+250, 190, 0, 0, 'See', '', [], tajLeftJustify);
  SpoolText(X, Y+500, 190, 0, 0, 'Schedule', '', [], tajLeftJustify);
  PrintCheckBox(X+1450, Y+275, 350, 25, Checked);
end;

procedure TCustomProcs.PrintExecutions(Y1, Y2: Integer; HasExecutions, CheckProp, CheckExec: Boolean);
var
  dy: Integer;
begin
  dy := (Y2 - Y1 - 3500) div 2;
  PrintLine(100, Y1+100, 100, Y2-100, 30);
  PrintLine(200, Y1+ 50, 200, Y1+ dy, 30);
  PrintLine(200, Y2- dy, 200, Y2- 50, 30);
  PrintLine(300,     Y1, 300, Y1+ dy, 30);
  PrintLine(300, Y2- dy, 300,     Y2, 30);
  PrintLine(400,     Y1, 400,     Y2, 30);
  PrintLine(500,     Y1, 500,     Y2, 30);
  PrintBMPs(125, Y1+ dy, 370,  Y2-dy, 'B11', OPAQUE);
  PrintRoundRect(15, Y1, 7750, Y2, 600, 30, True, True, False, True);
  if HasExecutions then
  begin
    PrintLine( 500, Y2-1700, 7750, Y2-1700, 30);
    SpoolText( 650, Y2-1600, 210, 0, 0, 'Executions', '', [fsBold], tajLeftJustify);
    SeeScheduleCheck(5800, Y2-850, CheckExec);
    Dec(Y2, 1700);
  end;
  SpoolText( 650, Y2-1250, 210, 0, 0, 'New Property Identifiers', '', [], tajLeftJustify);
  SeeScheduleCheck(5800, Y2-850, CheckProp);
end;

procedure TCustomProcs.PrintConsideration(Y1: Integer; const Prompt: String; RoundTopRight: Boolean);
var
  H: Integer;
begin                      
  if RoundTopRight then H := 1350
  else H := 1300;
  PrintRoundRect(7750, Y1, 20100, Y1+H, 600, 30, False, RoundTopRight, True, False);
  SpoolText( 7900, Y1+100, 210, 0, 0, Prompt, '', [fsBold], tajLeftJustify);
  SpoolText(15000, Y1+800, 210, 0, 0, 'Dollars $', '', [], tajLeftJustify);
end;

procedure TCustomProcs.Print_123(Y1: Integer; CheckReg, CheckLand, CheckProp: Boolean;
  NoPages: Integer; Memo: TCustomMemo);
begin
  PrintRoundRect( 7750,     Y1, 13750, Y1+ 850, 600, 30, True, True, False, False); {box 1}
  PrintRoundRect(13750,     Y1, 20100, Y1+ 850, 600, 30, True, True, True, False);  {box 2}
  PrintRoundRect( 7750, Y1+850, 20100, 3000, 600, 30, False, True, True, False);    {box 3}
  SpoolText( 7900, Y1+ 300, 210, 0, 0, '(1) Registry', '', [fsBold], tajLeftJustify);
  PrintCheckBox(9600, Y1+250, 350, 25, CheckReg);
  SpoolText(11400, Y1+ 300, 210, 0, 0, 'Land Titles', '', [fsBold], tajLeftJustify);
  PrintCheckBox(13020, Y1+250, 350, 25, CheckReg);
  SpoolText(13900, Y1+ 300, 210, 0, 0, '(2)', '', [fsBold], tajLeftJustify);
  SpoolText(14500, Y1+ 300, 210, 0, 0, 'Page 1', '', [], tajLeftJustify);
  SpoolText(SpoolText(15800,
    Y1+ 300, 210, 0, 0, 'of', '', [], tajLeftJustify),
    Y1+ 215, 295, 17350, 0, IntToStr(NoPages), 'TNR', [fsBold], tajCenter);
  SpoolText(17350, Y1+ 300, 210, 0, 0, 'pages', '', [], tajLeftJustify);
  SpoolText( 7900, Y1+ 950, 210, 0, 0, '(3) Property', '', [fsBold], tajLeftJustify);
  SpoolText( 8320, Y1+1200, 210, 0, 0, 'Identifier(s)', '', [fsBold], tajLeftJustify);
  SpoolText(11200, Y1+ 950, 210, 0, 0, 'Block', '', [], tajLeftJustify);
  SpoolText(13500, Y1+ 950, 210, 0, 0, 'Property', '', [], tajLeftJustify);
  PrintPropID(10100, Y1+1200, 8000, 0, Memo);
  SeeScheduleCheck(18200, Y1+1150, CheckProp);
end;

procedure TCustomProcs.PrintScaledMemoText(X, Y, W, CheckX, CheckY, InterLines: Integer;
  Memo: TCustomMemo; Squeeze, FlushIt: Boolean);
begin
end;

procedure TCustomProcs.PrintPropID(X, Y, W, InterLines: Integer; Memo: TCustomMemo);

function CheckLine(Line: String): Boolean;
begin
  Result := (Length(Line) > 0) and (Length(Line) <= 12) and
            ((Pos(#32, Line) > 0) or (Pos('-', Line) > 0));
end;

const
  X1: array [0..1] of Integer = (11200, 13500);
var
  Condition1, Condition2: Boolean;
  il, xx, yy, i, k : Integer;
  FontHeight       : Integer;
  S, Left, Right   : String;
begin
  if Memo = nil then Exit;
  if Memo.Lines.Count <= 0 then Exit;
  Condition1 := CheckLine(Memo.Lines[0]);
  Condition2 := (Memo.Lines.Count > 1) and CheckLine(Memo.Lines[1]);
  if not Condition1 and not Condition2 then
  begin
    PrintScaledMemoText(X, Y, W, 0, 0, InterLines, Memo, False, False);
    Exit;
  end;
  FontHeight := 0;
  for I := 0 to Memo.Lines.Count-1 do
  begin
    S := TrimTail(Memo.Lines[I]);
    if I < 2 then
      if CheckLine(S) then
      begin
        K := Pos(#32, S);
        if K = 0 then K := Pos('-', S);
        Left := Copy(S, 1, K-1);
        Right:= Copy(S, K+1, 255);
        SpoolText(X1[0], Y, 295, 0, 0, Left, 'TNR', [fsBold], tajLeftJustify);
        SpoolText(X1[1], Y, 295, 0, 0, Right, 'TNR', [fsBold], tajLeftJustify);
      end
      else
        SpoolText(X, Y, 295, X+W, 0, S, 'TNR', [fsBold], tajLeftJustify);
    if FontHeight = 0 then
        FontHeight := GetFontHeightNoInternalLeading;
    Inc(Y, FontHeight + InterLines);
  end;
end;

procedure TCustomProcs.PrintAmount(const Prompt: String; Y, H: Integer; HasRect: Boolean);
begin
  if HasRect then
    PrintRoundRect(7750, Y, 20100, Y+H, 600, 30, False, True, True, False);
  SpoolText( 7900, Y+100, 210, 0, 0, Prompt, '', [fsBold], tajLeftJustify);
  SpoolText(15000, Y+800, 210, 0, 0, 'Dollars $', '', [], tajLeftJustify);
end;

procedure TCustomProcs.PrintDescription(const Prompt: String; Y1, Y2, Y3: Integer; Memo: TCustomMemo);
begin
  PrintRoundRect(7750, Y1, 20100, Y2, 600, 30, False, True, True, False);
  SpoolText(7900, Y1+100, 210, 0, 0, Prompt, '', [fsBold], tajLeftJustify);
  PrintScaledMemoText(7800, Y3, 12200, 0, Y2-50, 0, Memo, False, False);
end;

procedure TCustomProcs.PrintFeesAndTax(Y1, Y2: Integer; S1, S2: String);
var
  dy, H: Integer;
begin
  dy := Y2 - Y1 - 3350;
  H  := (Y2 - Y1 - 450 - 700) div 4;
  PrintRoundRect(14000, Y1, 20100, Y2, 600, 30, False,True, True,True);
  PrintLine(14100, Y1, 14100, Y2-100, 30);
  PrintLine(14200, Y1, 14200, Y1+dy, 30);
  PrintLine(14300, Y1, 14300, Y1+dy, 30);
  PrintLine(14400, Y1, 14400, Y2, 30);
  PrintLine(14500, Y1, 14500, Y2, 30);
  PrintLine(14200, Y2-dy, 14200, Y2-50, 30);
  PrintLine(14300, Y2-dy, 14300, Y2, 30);
  PrintBMPs(14130, Y1+dy, 14350, Y2-dy, 'B11', OPAQUE);

  if S1 = '' then S1 := 'Fees';
  SpoolText(14000, Y1+100, 210, 20100, 0, S1, '', [fsBold], tajCenter);
  PrintLine(14500, Y1+450, 20100, Y1+450, 30);
  PrintLine(14500, Y1+450+H, 20100, Y1+450+H, 30);
  PrintLine(14500, Y1+450+2*H, 20100, Y1+450+2*H, 30);
  PrintLine(14500, Y1+450+3*H, 20100, Y1+450+3*H, 30);
  PrintLine(14500, Y1+450+4*H, 20100, Y1+450+4*H, 30);
  PrintLine(14500, Y1+550+4*H, 20100, Y1+550+4*H, 30);
  PrintLine(17050, Y1+450, 17050, Y2, 30);

  SpoolText(14600, Y1+750, 210, 0, 0, 'Registration Fee', '', [], tajLeftJustify);
  if S2 <> '' then
    SpoolText(14600, Y1+750+H, 210, 0, 0, S2, '', [], tajLeftJustify);
  SpoolText(14500, Y1+750+4*H, 210, 17050, 0, 'Total', '', [fsBold], tajCenter);
end;

procedure TCustomProcs.PrintEndorsement(X, Y: Integer; const Endorsement: String);
begin
  {SpoolText(SpoolText(X,
    Y, 150, 0, 1, 'Document prepared using', '', [], tajLeftJustify)+50,
    Y, 150, 0, 1, Endorsement, '', [fsBold,  fsItalic], tajLeftJustify);}
end;

procedure TCustomProcs.DocContains(X, Y: Integer; IsAorB, chkSketch, chkDescr, chkParties, chkOther: Boolean);
var
  R, H: Integer;
  TopLeft, TopRight, BottomRight, BottomLeft: Boolean;
begin
  if IsAorB then
  begin
    R := 12000;
    H := 1300;
    TopLeft     := True;
    TopRight    := False;
    BottomRight := False;
    BottomLeft  := True;
  end
  else
  begin
    R := 20100;
    H := 1400;
    TopLeft     := False;
    TopRight    := True;
    BottomRight := True;
    BottomLeft  := False;
  end;
  PrintRoundRect(X, Y, R, Y+H, 600, 30, TopLeft, TopRight, BottomRight, BottomLeft);

  Inc(X, 150);
  SpoolText(X, Y+100, 210, 0, 0, Format('(%d)', [6+Word(not IsAorB)]), '', [fsBold], tajLeftJustify);
  Inc(X, 400);
  SpoolText(X, Y+100, 210, 0, 0, 'This', '', [fsBold], tajLeftJustify);
  SpoolText(X, Y+350, 210, 0, 0, 'Document', '', [fsBold], tajLeftJustify);
  SpoolText(X, Y+600, 210, 0, 0, 'Contains', '', [fsBold], tajLeftJustify);
  Inc(X, 1550);
  SpoolText(X, Y+100, 210, 0, 0, '(a)', '', [], tajLeftJustify);
  Inc(X, 450);
  SpoolText(X, Y+100, 210, 0, 0, 'Redescription', '', [], tajLeftJustify);
  SpoolText(X, Y+450, 210, 0, 0, 'New Easement', '', [], tajLeftJustify);
  SpoolText(X, Y+800, 210, 0, 0, 'Plan/Sketch', '', [], tajLeftJustify);
  PrintCheckBox(X+2100, Y+650, 350, 25, chkSketch);
  Inc(X, 2800);
  PrintVDottedLine(X, Y, Y+H, 24, 0);
  Inc(X, 200);
  SpoolText(X, Y+100, 210, 0, 0, '(b) Schedule for:', '', [], tajLeftJustify);
  SpoolText(X, Y+800, 210, 0, 0, 'Description', '', [], tajLeftJustify);
  PrintCheckBox(X+1600, Y+650, 350, 25, chkDescr);
  Inc(X, 2700);
  SpoolText(X, Y+450, 210, 0, 0, 'Additional', '', [], tajLeftJustify);
  SpoolText(X, Y+800, 210, 0, 0, 'Parties', '', [], tajLeftJustify);
  PrintCheckBox(X+1450, Y+650, 350, 25, chkParties);
  Inc(X, 2100);
  SpoolText(X, Y+800, 210, 0, 0, 'Other', '', [], tajLeftJustify);
  PrintCheckBox(X+800, Y+650, 350, 25, chkOther);
end;

procedure TCustomProcs.PrintBox7Fees(Prompt: String);
begin
  PrintRoundRect(12000, 9400, 20100, 10700, 600, 30, False, True, True, False);
  SpoolText(12100, 9500, 210, 0, 0, '(7)', '', [fsBold], tajLeftJustify);
  SpoolText(12660, 9500, 210, 0, 0, Prompt, '', [fsBold], tajLeftJustify);
  SpoolText(12660, 9750, 210, 0, 0, 'Fee Simple', '', [], tajLeftJustify);
end;

procedure TCustomProcs.PrintAssessment(IsA: Boolean; Y1, Y2: Integer);
begin
  PrintRoundRect(15, Y1, 14000, Y2, 600, 30, not IsA, False, False, True);
  SpoolText(215, Y1+100, 210, 0, 0, Format('(%d) Assessement Roll Number', [15+Word(not IsA)]),
    '', [fsBold], tajLeftJustify);
  SpoolText(1400, Y1+400, 210, 0, 0, 'of Property', '', [fsBold], tajLeftJustify);
  SpoolText(5150, Y1+ 60, 180, 0, 0, 'Cty.', '', [], tajLeftJustify);
  SpoolText(5850, Y1+ 60, 180, 0, 0, 'Mun.', '', [], tajLeftJustify);
  SpoolText(6750, Y1+ 60, 180, 0, 0, 'Map', '', [], tajLeftJustify);
  SpoolText(7750, Y1+ 60, 180, 0, 0, 'Sub.', '', [], tajLeftJustify);
  SpoolText(9000, Y1+ 60, 180, 0, 0, 'Par.', '', [], tajLeftJustify);
  PrintVDottedLine(4980, Y1, Y2, 24, 0);
  PrintVDottedLine(5700, Y1, Y2, 24, 0);
  PrintVDottedLine(6450, Y1, Y2, 24, 0);
  PrintVDottedLine(7450, Y1, Y2, 24, 0);
  PrintVDottedLine(8470, Y1, Y2, 24, 0);
  PrintVDottedLine(9950, Y1, Y2, 24, 0);
end;

procedure TCustomProcs.PrintMunAddProp(X1, Y1, X2, Y2: Integer; Prompt1, Prompt2: String);
begin
  PrintRoundRect(X1, Y1, X2, Y2, 600, 30, True, False, True, True);
  SpoolText(X1+200, Y1+50, 210, 0, 0, Prompt1, '', [fsBold], tajLeftJustify);
  if Prompt2 = '' then Inc(Y1, 450) {??}
  else
  begin
    SpoolText(X1+635, Y1+350, 210, 0, 0, Prompt2, '', [fsBold], tajLeftJustify);
    Inc(Y1, 750);
  end;
end;

procedure TCustomProcs.PrintDocPrepBy(X1, Y1, X2, Y2: Integer; Prompt: String);
begin
  PrintRoundRect(X1, Y1, X2, Y2, 600, 30, False, False, True, True);
  SpoolText(X1+100, Y1+50, 210, 0, 0, Prompt, '', [fsBold], tajLeftJustify);
end;

procedure TCustomProcs.YMD_DateAndPrompts(Prompt1, Prompt2: String; X, Y1, Y2: Integer);
begin
  if Prompt1 <> '' then begin
    SpoolText(X, Y1, 210, X+2700, 0, Prompt1, '', [], tajCenter);
    Inc(Y1, 300);
  end;
  SpoolText(X, Y1, 210, X+2700, 0, Prompt2, '', [], tajCenter);
  Inc(Y1, 300);
  SpoolText(X+ 600, Y1, 210, 0, 0, 'Y', '', [], tajLeftJustify);
  SpoolText(X+1600, Y1, 210, 0, 0, 'M', '', [], tajLeftJustify);
  SpoolText(X+2300, Y1, 210, 0, 0, 'D', '', [], tajLeftJustify);
  Inc(Y1, 150);
  PrintVDottedLine(X, Y1, Y2, 24, 0);
  PrintVDottedLine(X+1400, Y1, Y2, 24, 0);
  PrintVDottedLine(X+2050, Y1, Y2, 24, 0);
end;

end.

