unit DlgPrint;

interface

uses
  Windows, SysUtils, Classes, Controls, Graphics, Forms, Menus, StdCtrls,
  Spin, Buttons, ExtCtrls,
  PCharLIB, EZ_Radio, EZ_Print;

type

  TDlgPrinterSettings = class(TForm)
    btnOk            : TBitBtn;
    btnCancel        : TBitBtn;
    GroupBox1        : TGroupBox;
    cbxPrinters      : TComboBox;
    btnGoTo          : TBitBtn;
    grpPaper         : TGroupBox;
    Label1           : TLabel;
    Label2           : TLabel;
    Label3           : TLabel;
    cbxPaperSize     : TComboBox;
    cbxPaperTray     : TComboBox;
    cbxQuality       : TComboBox;
    grpCopies        : TGroupBox;
    spinCopies       : TSpinEdit;
    grpOrient        : TRadioGroup;
    grpTrueType      : TEZ_RadioGroup;
    grpDuplex        : TRadioGroup;
    Label4           : TLabel;
    cbxDataType      : TComboBox;
    chkSave2DocDef   : TCheckBox;
    chkSaveAsDefault : TCheckBox;
    Label5: TLabel;
    ScrollBox1: TScrollBox;
    grpPages: TEZ_RadioGroup;
    Notebook1: TNotebook;
    Label6: TLabel;
    Label7: TLabel;
    spinFrom: TSpinEdit;
    spinTo: TSpinEdit;
    Label8: TLabel;
    Label9: TLabel;
    edtSelection: TEdit;
    lbxCustomSelect: TListBox;
    btnHelp: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Buf2Scr(Sender: TObject);
    procedure btnGoToClick(Sender: TObject);
    procedure PrinterINIChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure grpPagesClick(Sender: TObject);
    procedure edtSelectionChange(Sender: TObject);
  private
    { Private declarations }
    Printer: TEZ_Printer;
    FDevice, FDriver, FPort: PChar;
    DeviceMode: THandle;
    procedure Scr2Buf;
    procedure FreeCombos;
    function  UseCustomSelection: Boolean;
  public
    { Public declarations }
    constructor Create (AOwner: TComponent; APrinter: TEZ_Printer);
  end;

implementation

{$R *.DFM}

uses
  VCLTools;
  
constructor TDlgPrinterSettings.Create (AOwner: TComponent; APrinter: TEZ_Printer);
begin
  inherited Create(AOwner);
  Printer := APrinter;
end;

procedure TDlgPrinterSettings.FormCreate(Sender: TObject);
begin
  FDevice := StrAlloc(256);
  FDriver := StrAlloc(256);
  FPort   := StrAlloc(80);
end;

procedure TDlgPrinterSettings.FormDestroy(Sender: TObject);
begin
  Printer.OnINIChange := nil;
  StrDispose(FDevice);
  StrDispose(FDriver);
  StrDispose(FPort);
  FreeCombos;
end;

procedure TDlgPrinterSettings.FormShow(Sender: TObject);
begin
  Printer.OnIniChange := PrinterINIChange;
  PrinterINIChange(nil);
  CenterFormOverApp(Self);
end;

procedure TDlgPrinterSettings.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TDlgPrinterSettings.PrinterINIChange(Sender: TObject);
begin
  if Sender <> nil then
    Printer.CopyData(FDevice, FDriver, FPort, DeviceMode);
  cbxPrinters.Items.Assign(Printer.Printers);
  cbxPrinters.ItemIndex := Printer.PrinterIndex;
  Buf2Scr(Self);
end;

procedure TDlgPrinterSettings.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (ModalResult = mrCancel) or
	      ((cbxPrinters.ItemIndex > -1) and
               (cbxPaperTray.ItemIndex > -1) and
               ((grpPages.ItemIndex < 2) or
                 ((UseCustomSelection and (lbxCustomSelect.SelCount > 0)) or
                 (not UseCustomSelection and (edtSelection.Text <> '')))
               ));
  if not CanClose then ModalResult := mrCancel
  else
  begin
    if ModalResult = mrOk then
      Scr2Buf;
    if DeviceMode <> 0 then
      GlobalFree(DeviceMode);
    DeviceMode := 0;
  end;
end;

procedure TDlgPrinterSettings.FreeCombos;

  procedure cbxFree(C: TComboBox);
  var I: Integer;
  begin
    with C, Items do
      if Count > 0 then
	for I := 0 to Count - 1 do
          Objects[I].Free;
    C.Clear;
  end;

begin
  cbxFree(cbxPaperSize);
  cbxFree(cbxPaperTray);
  cbxFree(cbxQuality);
  Application.ProcessMessages;
end;

procedure TDlgPrinterSettings.Buf2Scr(Sender: TObject);
begin
  Enabled := False; // we don't want user to interact during setup
  try
    Printer.PrinterIndex := cbxPrinters.ItemIndex;
    Printer.CopyData(FDevice, FDriver, FPort, DeviceMode);
    btnGoTo.Enabled := not (dlgDisableDefault in Printer.dlgPrintOptions);
    cbxPrinters.Enabled := btnGoTo.Enabled;

    // set duplex group
    grpDuplex.Enabled := Printer.CanPrintDuplex and
                         not (dlgDisableDuplex in Printer.dlgPrintOptions);

    if not grpDuplex.Enabled then grpDuplex.ItemIndex := -1
    else grpDuplex.ItemIndex := pred(Printer.Duplex);

    // set TrueType group and radio btns
    grpTrueType.Enabled :=
      Printer.CanHandleTTOption(DMTT_BITMAP) or
      Printer.CanHandleTTOption(DMTT_DOWNLOAD) or
      Printer.CanHandleTTOption(DMTT_SUBDEV) or
      Printer.CanHandleTTOption(DMTT_DOWNLOAD_OUTLINE);
    if not grpTrueType.Enabled then grpTrueType.ItemIndex := -1
    else
    begin
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_BITMAP), [0]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_DOWNLOAD), [1]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_SUBDEV), [2]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_DOWNLOAD_OUTLINE), [3]);
      grpTrueType.ItemIndex := ord(Printer.TrueTypeMode);
    end;

    // set Orientation group
    grpOrient.ItemIndex := ord(Printer.Orientation);
    grpOrient.Enabled := not (dlgDisableOrient in Printer.dlgPrintOptions);

    // set # of Copies
    spinCopies.Value := Printer.Copies;

    // set Paper Size, Tray, Quality and set DataType
    FreeCombos;
    Printer.SetPaperSizes(cbxPaperSize);
    Printer.SetPaperTrays(cbxPaperTray);
    Printer.SetPaperQuality(cbxQuality);

    cbxPaperSize.Enabled := not (dlgDisableSize in Printer.dlgPrintOptions);

    // set Datatype (EMF, RAW, etc)
    cbxDataType.Enabled := (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
                            Printer.Is_EMF_COMPLIANT;
    if not cbxDataType.Enabled then cbxDataType.Color := clBtnFace
    else cbxDataType.Color := clWindow;
    Printer.SetDataType(cbxDataType);
    Label5.Visible := (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
                       not Printer.Is_EMF_COMPLIANT;

    with Printer do
    begin
      spinFrom.MaxValue := ToPage;
      spinTo.MaxValue   := ToPage;
      spinFrom.MinValue := FromPage;
      spinTo.MinValue   := FromPage;
      spinTo.Value      := ToPage;
      spinFrom.Value    := FromPage;
      if ToPage = 1 then
        grpPages.RadEnable(False, [1, 2]);
    end;

    if Assigned(Printer.OnSetCustomPages) then
      Printer.OnSetCustomPages(Printer, lbxCustomSelect.Items);
    grpPagesClick(grpPages);
    if UseCustomSelection then grpPages.RadEnable(False, [1]);

  finally
    Enabled := True; // let user interact with ctrls
  end;
end;

procedure TDlgPrinterSettings.Scr2Buf;
var
  I: Integer;
begin
  // set Copies
  Printer.Copies := spinCopies.Value;
  // set PaperSize
  with cbxPaperSize, Items do if ItemIndex <> -1 then
    Printer.PaperSize     := TWordObj(Objects[ItemIndex]).CurWord;
  // set DefaultSource (paper tray)
  with cbxPaperTray, Items do if ItemIndex <> -1 then
    Printer.DefaultSource := TWordObj(Objects[ItemIndex]).CurWord;
  // set PrintQuality
  with cbxQuality, Items do if ItemIndex <> -1 then
  begin
    Printer.YResolution := TResObj(Objects[ItemIndex]).HorzRes;
    Printer.PrintQuality := TResObj(Objects[ItemIndex]).HorzRes;
  end;
  // set PrinterDataType
  if Win32Platform = VER_PLATFORM_WIN32_WINDOWS then
    with cbxDataType do if ItemIndex <> -1 then
      if Pos('Current', Text) = 0  then
        Printer.PrinterDataType := Text;
  // set Duplex
  if grpDuplex.Enabled then if grpDuplex.ItemIndex <> -1 then
    Printer.Duplex := succ(grpDuplex.ItemIndex);
  // set TrueTypeMode
  if grpTrueType.Enabled then if grpTrueType.ItemIndex <> -1 then
    Printer.TrueTypeMode := TPrinterTrueType(grpTrueType.ItemIndex);
  // set Orientation
  if grpOrient.ItemIndex <> -1 then
    Printer.Orientation := TPrinterOrientation(grpOrient.ItemIndex);

  if chkSave2DocDef.Checked then Printer.SaveToDefaults;
  if chkSaveAsDefault.Checked then Printer.SavePrinterAsDefault;

  if grpPages.ItemIndex < 2 then
  begin
    MyStrDispose(Printer.pageSelected);
    if Assigned(Printer.CustomPagesSelected) then Printer.CustomPagesSelected.Free;
    Printer.CustomPagesSelected := nil;
  end;

  if UseCustomSelection and (grpPages.ItemIndex <> 1) then
  begin
    if (grpPages.ItemIndex = 0) then  // All
      for I := 0 to lbxCustomSelect.Items.Count - 1 do // select
        lbxCustomSelect.Selected[I] := True;           // all pages
    Printer.CustomPagesSelected := TStringList.Create;
    for I := 0 to lbxCustomSelect.Items.Count - 1 do
      if lbxCustomSelect.Selected[I] then
        Printer.CustomPagesSelected.Add(lbxCustomSelect.Items[I]);
  end
  else
  begin
    Printer.ToPage   := SpinTo.Value;
    Printer.FromPage := SpinFrom.Value;
  end;

  // need below cuz of posible message Broadcasted
  Application.ProcessMessages;
end;

procedure TDlgPrinterSettings.btnGoToClick(Sender: TObject);
begin
  // select Default Printer
  Printer.PrinterIndex := -1;
  // reset data
  PrinterINIChange(Printer);
end;

function TDlgPrinterSettings.UseCustomSelection: Boolean;
begin
  Result := lbxCustomSelect.Items.Count > 0;
end;

procedure TDlgPrinterSettings.grpPagesClick(Sender: TObject);
var
  Enable: Boolean;
begin
  Enable := grpPages.ItemIndex > 0;
  spinFrom.Enabled := Enable;
  spinTo.Enabled := Enable;
  if Enable then
  begin
    spinFrom.Color := clWindow;
    spinTo.Color := clWindow;
  end
  else
  begin
    spinFrom.Color := clBtnFace;
    spinTo.Color := clBtnFace;
  end;
  case grpPages.ItemIndex of
    0, 1: Notebook1.PageIndex := 0;
    2: if not UseCustomSelection then Notebook1.PageIndex := 1
       else Notebook1.PageIndex := 2;
  end;

  if grpPages.ItemIndex < 2 then
  begin
    MyStrDispose(Printer.pageSelected);
    if Assigned(Printer.CustomPagesSelected) then Printer.CustomPagesSelected.Free;
    Printer.CustomPagesSelected := nil;
    btnOk.Enabled := True;
  end
  else
  begin
    if not UseCustomSelection then
      edtSelection.Text := StrPasEx(Printer.pageSelected);
    btnOk.Enabled :=
      (UseCustomSelection and (lbxCustomSelect.SelCount > 0)) or
      (not UseCustomSelection and (edtSelection.Text <> ''))
  end;
end;

procedure TDlgPrinterSettings.edtSelectionChange(Sender: TObject);
var
  S: String;
  I: Integer;
  Enbld: Boolean;
begin
  Enbld := False;
  S := edtSelection.Text;
  if S <> '' then
  begin
    I := 1;
    repeat
      if not (S[I] in ['0'..'9', ',', '''', '-']) then Delete(S, I, 1)
      else Inc(I);
    until I > Length(S);
    if S <> '' then
    begin
      MyStrDispose(Printer.pageSelected);
      Printer.pageSelected := StrPDup(S);
      with Printer do
        Enbld := CompilePageSelection(pageSelected, FromPage, ToPage);
      if not Enbld then MyStrDispose(Printer.pageSelected);
    end;
  end;
  btnOk.Enabled := Enbld;
end;


end.

