{*******************************************************}
{                                                       }
{       EZ_Radio.pas                                    }
{       ------------                                    }
{       ADV32 Compiled & Tested                         }
{       under Windows 95, '98 and NT40                  }
{       Copyright  1997-2000, Lucian Radulescu         }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{                                                       }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit EZ_Radio;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type

  TEZ_CustomRadioGroup = class(TCustomGroupBox)
  private
    FButtons: TList;
    FItems: TStrings;
    FItemIndex: Integer;
    FColumns: Integer;
    FReading: Boolean;
    FUpdating: Boolean;

    FAtLeft: Integer;
    FAtTop: Integer;
    FAtRight: Integer;
    FBaseLine: Integer;
    FFontRadio: TFont;
    FShowBaseLine: Boolean;
    FShowTopLine: Boolean;
    procedure ArrangeButtons;
    procedure ButtonClick(Sender: TObject);
    procedure ItemsChange(Sender: TObject);
    procedure SetButtonCount(Value: Integer);
    procedure SetColumns(Value: Integer);
    procedure SetItemIndex(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure UpdateButtons;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;

    procedure SetAtLeft(Value: Integer);
    procedure SetAtTop(Value: Integer);
    procedure SetAtRight(Value: Integer);
    procedure SetBaseLine(Value: Integer);
    procedure SetFontRadio(Value: TFont);
    procedure SetShowBaseLine(Value: Boolean);
    procedure SetShowTopLine(Value: Boolean);
    function  ReservedTop: Integer;
  protected
    procedure Loaded; override;
    procedure Paint; override;
    procedure ReadState(Reader: TReader); override;
    function CanModify: Boolean; virtual;
    property Columns: Integer read FColumns write SetColumns default 1;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
    property Items: TStrings read FItems write SetItems;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {$IFDEF ATLEAST_D4}
    procedure FlipChildren(AllLevels: Boolean); override;
    {$ENDIF ATLEAST_D4}
  published
    { Published declarations }
    property AtLeft: Integer read FAtLeft write SetAtLeft default 0;
    property AtRight: Integer read FAtRight write SetAtRight default 0;
    property AtTop: Integer read FAtTop write SetAtTop;
    property BaseLine: Integer read FBaseLine write SetBaseLine default 0;
    property FontRadio: TFont read FFontRadio write SetFontRadio;
    property ShowBaseLine: Boolean read FShowBaseLine write SetShowBaseLine default False;
    property ShowTopLine: Boolean read FShowTopLine write SetShowTopLine default False;
  end;

  { TEZ_RadioGroup }
  TEZ_RadioGroup = class(TEZ_CustomRadioGroup)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure RadEnable(Enable: Boolean; Who: array of Integer); virtual;
  published
    { Published declarations }
    property Align;
    property AtLeft;
    property AtRight;
    property AtTop;
    property BaseLine;
    {$IFDEF ATLEAST_D4}
    property Anchors;
    property BiDiMode;
    {$ENDIF ATLEAST_D4}
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    {$IFDEF ATLEAST_D4}
    property DragKind;
    {$ENDIF ATLEAST_D4}
    property DragMode;
    property Enabled;
    property Font;
    property ItemIndex;
    property Items;
    {$IFDEF ATLEAST_D4}
    property Constraints;
    property ParentBiDiMode;
    {$ENDIF ATLEAST_D4}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowBaseLine;
    property ShowHint;
    property ShowTopLine;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    {$IFDEF VER130}
    property OnContextPopup;
    {$ENDIF VER130}
    property OnDragDrop;
    property OnDragOver;
    {$IFDEF ATLEAST_D4}
    property OnEndDock;
    {$ENDIF ATLEAST_D4}
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    {$IFDEF ATLEAST_D4}
    property OnStartDock;
    {$ENDIF ATLEAST_D4}
    property OnStartDrag;
  end;

procedure Register;

implementation

{.$R *.R32}

procedure Register;
begin
  RegisterComponents('JeeLIB', [TEZ_RadioGroup]);
end;

{ TGroupButton }

type
  TGroupButton = class(TRadioButton)
  private
    FInClick: Boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    constructor InternalCreate(RadioGroup: TEZ_CustomRadioGroup);
    destructor Destroy; override;
  end;

constructor TGroupButton.InternalCreate(RadioGroup: TEZ_CustomRadioGroup);
begin
  inherited Create(RadioGroup);
  RadioGroup.FButtons.Add(Self);
  Visible := False;
  Enabled := RadioGroup.Enabled;
  Font.Assign(RadioGroup.FFontRadio);
  ParentShowHint := False;
  OnClick := RadioGroup.ButtonClick;
  Parent := RadioGroup;
end;

destructor TGroupButton.Destroy;
begin
  TEZ_CustomRadioGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

procedure TGroupButton.CNCommand(var Message: TWMCommand);
begin
  if not FInClick then
  begin
    FInClick := True;
    try
      if ((Message.NotifyCode = BN_CLICKED) or
          (Message.NotifyCode = BN_DOUBLECLICKED)) and
        TEZ_CustomRadioGroup(Parent).CanModify then
        inherited;
    except
      Application.HandleException(Self);
    end;
    FInClick := False;
  end;
end;

procedure TGroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TEZ_CustomRadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
  begin
    if not TEZ_CustomRadioGroup(Parent).CanModify then Key := #0;
  end;
end;

procedure TGroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TEZ_CustomRadioGroup(Parent).KeyDown(Key, Shift);
end;

{ TEZ_CustomRadioGroup }
constructor TEZ_CustomRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csSetCaption, csDoubleClicks, csReplicatable];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
  FAtLeft := 0;
  FAtRight := 0;
  FBaseLine := 0;
  FFontRadio := TFont.Create;
  FShowBaseLine := False;
  FShowTopLine := False;
end;

destructor TEZ_CustomRadioGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange := nil;
  FFontRadio.Free;
  FItems.Free;
  FButtons.Free;
  inherited Destroy;
end;

{$IFDEF ATLEAST_D4}
procedure TEZ_CustomRadioGroup.FlipChildren(AllLevels: Boolean);
begin
  { The radio buttons are flipped using BiDiMode }
end;
{$ENDIF ATLEAST_D4}

procedure TEZ_CustomRadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
  ALeft: Integer;
  {$IFDEF ATLEAST_D3}DeferHandle: THandle;{$ENDIF}
begin
  if (FButtons.Count <> 0) and not FReading then
  begin
    DC := GetDC(0);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol := (FButtons.Count + FColumns - 1) div FColumns;
    ButtonWidth := (Width - 10 - FAtLeft - FAtRight) div FColumns;
    I := Height - Metrics.tmHeight - FBaseLine - FAtTop - 5;
    ButtonHeight := I div ButtonsPerCol;
    TopMargin := FAtTop + Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
    {$IFDEF ATLEAST_D3}DeferHandle := BeginDeferWindowPos(FButtons.Count);{$ENDIF}
    try
      for I := 0 to FButtons.Count - 1 do
        with TGroupButton(FButtons[I]) do
        begin
	  {$IFDEF ATLEAST_D4}BiDiMode := Self.BiDiMode;{$ENDIF ATLEAST_D4}
          ALeft := (I div ButtonsPerCol) * ButtonWidth + 8 + FAtLeft;
	  {$IFDEF ATLEAST_D4}
	  if UseRightToLeftAlignment then
	    ALeft := Self.ClientWidth - ALeft - ButtonWidth;
	  {$ENDIF ATLEAST_D4}  
          {$IFDEF ATLEAST_D3}
          DeferHandle := DeferWindowPos(DeferHandle, Handle, 0,
            ALeft,
            (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
            ButtonWidth, ButtonHeight,
            SWP_NOZORDER or SWP_NOACTIVATE);
          {$ELSE}
          SetBounds((I div ButtonsPerCol) * ButtonWidth + 8 + FAtLeft,
                    (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
                      ButtonWidth, ButtonHeight);
          {$ENDIF}
          Visible := True;
        end;
    finally
    {$IFDEF ATLEAST_D3}EndDeferWindowPos(DeferHandle);{$ENDIF}
    end;
  end;
end;

procedure TEZ_CustomRadioGroup.SetFontRadio(Value: TFont);
var
  I: Integer;
begin
  FFontRadio.Assign(Value);
  if FButtons.Count = 0 then Exit;
  for I := 0 to FButtons.Count - 1 do
    TGroupButton(FButtons[I]).Font.Assign(Value);
  Refresh;
end;

procedure TEZ_CustomRadioGroup.Paint;

procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
begin
  with Canvas do
  begin
    Pen.Color := C;
    MoveTo(X1, Y1);
    LineTo(X2, Y2);
  end;
end;

var
  H, W: Integer;
  Color1, Color2: TColor;
  R: TRect;
begin
  if FBaseLine > 0 then
    with Canvas do
    begin
      Pen.Width := 1;
      H := Height - FBaseLine;
      W := Width - 2;
      R := Rect(2, H, W, Height-2);
      Brush.Color := Color;
      FillRect(R);
      if FShowBaseLine then
      begin
        Color1 := clBtnShadow;
        Color2 := clBtnHighlight;
      end
      else
      begin
        Color1 := Color;
        Color2 := Color;
      end;
      BevelLine(Color1, 2, H, W, H);
      BevelLine(Color2, 2, H + 1, W, H + 1);
    end;

  if FAtTop > 0 then
    with Canvas do
    begin
      Pen.Width := 1;
      W := Width - 2;
      R := Rect(2, FAtTop-5, W, FAtTop + 1);
      Brush.Color := Color;
      FillRect(R);
      if FShowTopLine then
      begin
        Color1 := clBtnShadow;
        Color2 := clBtnHighlight;
      end
      else
      begin
        Color1 := Color;
        Color2 := Color;
      end;
      BevelLine(Color1, 2, FAtTop, W, FAtTop);
      BevelLine(Color2, 2, FAtTop + 1, W, FAtTop + 1);
    end;
  inherited Paint;
end;

procedure TEZ_CustomRadioGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
  begin
    FItemIndex := FButtons.IndexOf(Sender);
    {$IFDEF ATLEAST_D3}Changed;{$ENDIF}
    Click;
  end;
end;

procedure TEZ_CustomRadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
  begin
    if FItemIndex >= FItems.Count then FItemIndex := FItems.Count - 1;
    UpdateButtons;
  end;
end;

procedure TEZ_CustomRadioGroup.Loaded;
begin
  inherited Loaded;
  ArrangeButtons;
end;

procedure TEZ_CustomRadioGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;

procedure TEZ_CustomRadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TGroupButton.InternalCreate(Self);
  while FButtons.Count > Value do TGroupButton(FButtons.Last).Free;
end;

procedure TEZ_CustomRadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if FColumns <> Value then
  begin
    FColumns := Value;
    ArrangeButtons;
    {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
  end;
end;

procedure TEZ_CustomRadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex := Value else
  begin
    if Value < -1 then Value := -1;
    if Value >= FButtons.Count then Value := FButtons.Count - 1;
    if FItemIndex <> Value then
    begin
      if FItemIndex >= 0 then
        TGroupButton(FButtons[FItemIndex]).Checked := False;
      FItemIndex := Value;
      if FItemIndex >= 0 then
        TGroupButton(FButtons[FItemIndex]).Checked := True;
    end;
  end;
end;

procedure TEZ_CustomRadioGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TEZ_CustomRadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
    TGroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then
  begin
    FUpdating := True;
    TGroupButton(FButtons[FItemIndex]).Checked := True;
    FUpdating := False;
  end;
  ArrangeButtons;
  {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
end;

procedure TEZ_CustomRadioGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I := 0 to FButtons.Count - 1 do
    TGroupButton(FButtons[I]).Enabled := Enabled;
end;

procedure TEZ_CustomRadioGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AtTop := ReservedTop;
  ArrangeButtons;
  {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
end;

procedure TEZ_CustomRadioGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
  {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
end;

function TEZ_CustomRadioGroup.CanModify: Boolean;
begin
  Result := True;
end;

function TEZ_CustomRadioGroup.ReservedTop: Integer;
{var
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;}
begin
  {DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  Result := Metrics.tmHeight - Metrics.tmInternalLeading;}
  Result := 0;
end;

procedure TEZ_CustomRadioGroup.SetAtLeft(Value: Integer);
begin
  if FAtLeft <> Value then
    if (Value >= 0) and (Value < Width - FAtRight - 20) then
    begin
      FAtLeft := Value;
      ArrangeButtons;
      {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
    end;
end;

procedure TEZ_CustomRadioGroup.SetAtTop(Value: Integer);
begin
  if FAtTop <> Value then
    if Value = 0 then
    begin
      FAtTop := ReservedTop;
      ShowTopLine := False;
    end;
    if (Value >= ReservedTop) and (Value < Height - FBaseLine - 20)  then
    begin
      FAtTop := Value;
      ArrangeButtons;
      {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
    end;
end;

procedure TEZ_CustomRadioGroup.SetAtRight(Value: Integer);
begin
  if FAtRight <> Value then
    if (Value >= 0) and (Value < Width - FAtLeft - 20) then
    begin
      FAtRight := Value;
      ArrangeButtons;
      {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
    end;
end;

procedure TEZ_CustomRadioGroup.SetBaseLine(Value: Integer);
begin
  if FBaseLine <> Value then
    if Value = 0 then
    begin
      FBaseLine := Value;
      ShowBaseLine := False;
    end
    else
    if Value > 1 then
      if Value < Height - FAtTop - 20 then
      begin
        FBaseLine := Value;
        ArrangeButtons;
        {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
      end;
end;

procedure TEZ_CustomRadioGroup.SetShowBaseLine(Value: Boolean);
begin
  if Value and (FBaseLine = 0) then Exit;
  FShowBaseLine := Value;
  ArrangeButtons;
  {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
end;

procedure TEZ_CustomRadioGroup.SetShowTopLine(Value: Boolean);
begin
  if Value and (FAtTop = 0) then Exit;
  FShowTopLine := Value;
  ArrangeButtons;
  {$IFDEF ATLEAST_D3}Invalidate{$ENDIF};
end;

procedure TEZ_RadioGroup.RadEnable(Enable: Boolean; Who: array of Integer);
var
  I: Integer;
begin
  for I := Low(Who) to High(Who) do
    TGroupButton(FButtons[Who[I]]).Enabled := Enable;
end;

end.

