unit MathTool;

interface

uses
  Windows,
  Printers,
  EZ_Print;

type

  TUnitsSet = (
    usInch,
    usMM,
    usThouMM,
    usTwips,
    usPoints,
    usPixScreen,
    usPixPrinter
  );

const

  UnitValues : array [ TUnitsSet ] of single = (
        1.0,     // 1" = 1"
       25.4,     // 1" = 25.4 mm
    25400.0,     // 1" = 25400 thouMM
     1440.0,     // 1" = 1440 twips
       72.0,     // 1" = 72 points
       96.0,     // 1" = 96 pixels   set on initialzation
        0.0      // DO NOT USE THIS DIRECTLY
  );

function ConvertUnits (
  in_Value  : single;    // convert in_Value
  in_Units  : TUnitsSet; // from in_Units
  out_Units : TUnitsSet; // into out_Units
  aPrinter  : TEZ_Printer// it will be used if at least on unit is usPixPrinter
): single;

procedure ResetUnitValues;

implementation

uses
  Consts;

function ConvertUnits (
  in_Value  : single;
  in_Units  : TUnitsSet;
  out_Units : TUnitsSet;
  aPrinter  : TEZ_Printer
): single;
begin
  if (( in_Units = usPixPrinter ) or ( out_Units = usPixPrinter )) then
    if Assigned( aPrinter ) then
      UnitValues [ usPixPrinter ] := GetDeviceCaps ( aPrinter.Handle, LOGPIXELSX )
    else
      UnitValues [ usPixPrinter ] := GetDeviceCaps ( Printer.Handle, LOGPIXELSX );

  Result := in_Value * UnitValues [ out_Units ] / UnitValues [ in_Units ];
end;

procedure ResetUnitValues;
var                                                          
  DC: HDC;
begin
  DC := GetDC ( 0 );
  UnitValues [ usPixScreen ] := GetDeviceCaps ( DC, LOGPIXELSX );
  ReleaseDC ( 0, DC );
end;

initialization
  ResetUnitValues;
end.
