unit DlgPrint;

interface

uses
  Windows, SysUtils, Classes, Controls, Graphics, Forms, Menus, StdCtrls,
  Spin, Buttons, ExtCtrls, EZ_Radio, EZ_Print;

type

  TDlgPrinterSettings = class(TForm)
    btnOk            : TBitBtn;
    btnCancel        : TBitBtn;
    GroupBox1        : TGroupBox;
    cbxPrinters      : TComboBox;
    btnGoTo          : TBitBtn;
    grpPaper         : TGroupBox;
    Label1           : TLabel;
    Label2           : TLabel;
    Label3           : TLabel;
    cbxPaperSize     : TComboBox;
    cbxPaperTray     : TComboBox;
    cbxQuality       : TComboBox;
    grpCopies        : TGroupBox;
    spinCopies       : TSpinEdit;
    grpOrient        : TRadioGroup;
    grpTrueType: TEZ_RadioGroup;
    grpDuplex        : TRadioGroup;
    Label4           : TLabel;
    cbxDataType      : TComboBox;
    chkSave2DocDef   : TCheckBox;
    chkSaveAsDefault : TCheckBox;
    Label5: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Buf2Scr(Sender: TObject);
    procedure btnGoToClick(Sender: TObject);
    procedure PrinterINIChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    Printer: TEZ_Printer;
    FDevice, FDriver, FPort: PChar;
    DeviceMode: THandle;
    procedure Scr2Buf;
    procedure FreeCombos;
  public
    { Public declarations }
    constructor Create (AOwner: TComponent; APrinter: TEZ_Printer);
  end;

implementation

{$R *.DFM}

constructor TDlgPrinterSettings.Create (AOwner: TComponent; APrinter: TEZ_Printer);
begin
  inherited Create(AOwner);
  Printer := APrinter;
end;

procedure TDlgPrinterSettings.FormCreate(Sender: TObject);
begin
  FDevice := StrAlloc(256);
  FDriver := StrAlloc(256);
  FPort   := StrAlloc(256);
end;

procedure TDlgPrinterSettings.FormDestroy(Sender: TObject);
begin
  Printer.OnINIChange := nil;
  StrDispose(FDevice);
  StrDispose(FDriver);
  StrDispose(FPort);
  FreeCombos;
end;

procedure TDlgPrinterSettings.FormShow(Sender: TObject);
begin
  Printer.OnIniChange := PrinterINIChange;
  PrinterINIChange(nil);
end;

procedure TDlgPrinterSettings.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TDlgPrinterSettings.PrinterINIChange(Sender: TObject);
begin
  if Sender <> nil then
    Printer.CopyData(FDevice, FDriver, FPort, DeviceMode);
  cbxPrinters.Items.Assign(Printer.Printers);
  cbxPrinters.ItemIndex := Printer.PrinterIndex;
  Buf2Scr(Self);
end;

procedure TDlgPrinterSettings.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (ModalResult = mrCancel) or
	      ((cbxPrinters.ItemIndex > -1) and (cbxPaperTray.ItemIndex > -1));
  if not CanClose then ModalResult := mrCancel
  else
  begin
    if ModalResult = mrOk then
      Scr2Buf;
    if DeviceMode <> 0 then
      GlobalFree(DeviceMode);
    DeviceMode := 0;
  end;
end;

procedure TDlgPrinterSettings.FreeCombos;

  procedure cbxFree(C: TComboBox);
  var I: Integer;
  begin
    with C, Items do
      if Count > 0 then
	for I := 0 to Count - 1 do
          Objects[I].Free;
    C.Clear;
  end;

begin
  cbxFree(cbxPaperSize);
  cbxFree(cbxPaperTray);
  cbxFree(cbxQuality);
  Application.ProcessMessages;
end;

procedure TDlgPrinterSettings.Buf2Scr(Sender: TObject);
begin
  Enabled := False; // we don't want user to interact during setup
  try
    Printer.PrinterIndex := cbxPrinters.ItemIndex;
    Printer.CopyData(FDevice, FDriver, FPort, DeviceMode);

    // set duplex group
    grpDuplex.Enabled := Printer.CanPrintDuplex;
    if not grpDuplex.Enabled then grpDuplex.ItemIndex := -1
    else grpDuplex.ItemIndex := pred(Printer.Duplex);

    // set TrueType group and radio btns
    grpTrueType.Enabled :=
      Printer.CanHandleTTOption(DMTT_BITMAP) or
      Printer.CanHandleTTOption(DMTT_DOWNLOAD) or
      Printer.CanHandleTTOption(DMTT_SUBDEV) or
      Printer.CanHandleTTOption(DMTT_DOWNLOAD_OUTLINE);
    if not grpTrueType.Enabled then grpTrueType.ItemIndex := -1
    else
    begin
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_BITMAP), [0]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_DOWNLOAD), [1]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_SUBDEV), [2]);
      grpTrueType.RadEnable(Printer.CanHandleTTOption(DMTT_DOWNLOAD_OUTLINE), [3]);
      grpTrueType.ItemIndex := ord(Printer.TrueTypeMode);
    end;

    // set Orientation group
    grpOrient.ItemIndex := ord(Printer.Orientation);

    // set # of Copies
    spinCopies.Value := Printer.Copies;

    // set Paper Size, Tray, Quality and set DataType
    FreeCombos;
    Printer.SetPaperSizes(cbxPaperSize);
    Printer.SetPaperTrays(cbxPaperTray);
    Printer.SetPaperQuality(cbxQuality);

    // set Datatype (EMF, RAW, etc)
    cbxDataType.Enabled := (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
                            Printer.Is_EMF_COMPLIANT;
    if not cbxDataType.Enabled then cbxDataType.Color := clBtnFace
    else cbxDataType.Color := clWindow;
    Printer.SetDataType(cbxDataType);
    Label5.Visible := (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
                       not Printer.Is_EMF_COMPLIANT;
    
  finally
    Enabled := True; // let user interact with ctrls
  end;
end;

procedure TDlgPrinterSettings.Scr2Buf;
begin
  // set Copies
  Printer.Copies := spinCopies.Value;
  // set PaperSize
  with cbxPaperSize, Items do if ItemIndex <> -1 then
    Printer.PaperSize     := TWordObj(Objects[ItemIndex]).CurWord;
  // set DefaultSource (paper tray)
  with cbxPaperTray, Items do if ItemIndex <> -1 then
    Printer.DefaultSource := TWordObj(Objects[ItemIndex]).CurWord;
  // set PrintQuality
  with cbxQuality, Items do if ItemIndex <> -1 then
  begin
    Printer.YResolution := TResObj(Objects[ItemIndex]).HorzRes;
    Printer.PrintQuality := TResObj(Objects[ItemIndex]).HorzRes;
  end;
  // set PrinterDataType
  if Win32Platform = VER_PLATFORM_WIN32_WINDOWS then
    with cbxDataType do if ItemIndex <> -1 then
      if Pos('Current', Text) = 0  then
        Printer.PrinterDataType := Text;
  // set Duplex
  if grpDuplex.Enabled then if grpDuplex.ItemIndex <> -1 then
    Printer.Duplex := succ(grpDuplex.ItemIndex);
  // set TrueTypeMode
  if grpTrueType.Enabled then if grpTrueType.ItemIndex <> -1 then
    Printer.TrueTypeMode := TPrinterTrueType(grpTrueType.ItemIndex);
  // set Orientation
  if grpOrient.ItemIndex <> -1 then
    Printer.Orientation := TPrinterOrientation(grpOrient.ItemIndex);

  if chkSave2DocDef.Checked then Printer.SaveToDefaults;
  if chkSaveAsDefault.Checked then Printer.SavePrinterAsDefault;

  // need below cuz of posible message Broadcasted 
  Application.ProcessMessages;
end;

procedure TDlgPrinterSettings.btnGoToClick(Sender: TObject);
begin
  // select Default Printer
  Printer.PrinterIndex := -1;
  // reset data
  PrinterINIChange(Printer);
end;

end.

