{*******************************************************}
{                                                       }
{      Couple functions using VCL - Implementation Part }
{      -----------------------------------------------  }
{      ADV Compiled                                     }
{      Copyright (C), Lucian Radulescu                  }
{      E-mail:    ljee@interlog.com                     }
{      Home Page: http://www.interlog.com/~ljee         }
{                                                       }
{*******************************************************}

unit VCLTools;

{$I VCLTOOLS.INT}

implementation

uses
  PCharLIB, FileTool, WinTools;

{$IFNDEF WIN32}
procedure SwitchToThisWindow; external 'USER' Index 172;

procedure ShowApplication;
var
  Hnd: HWND;
begin
  Hnd := Application.Handle;
  if not IsNT then SwitchToThisWindow(Hnd, True)
  else
  begin
    if not IsIconic(Hnd) then Application.Minimize;
    ShowWindow(Hnd, SW_RESTORE);
  end;
end;
{$ENDIF WIN32}

function MemoGetTextLen(Memo: TCustomEdit): Cardinal;
begin
  Result := SendMessage(Memo.Handle, WM_GETTEXTLENGTH, 0, 0);
end;

function UpdateMemoFromBuffer(Memo: TCustomEdit; Buffer: PChar): Longint;
begin
  Result := SendMessage(Memo.Handle, WM_SETTEXT, 0, Longint(Buffer));
end;

function GetMemo2Buffer(Memo: TCustomEdit): PChar;
var
  Size: Cardinal;
begin
  Result := nil;
  Size := MemoGetTextLen(Memo);
  if Size = 0 then Exit;
  Inc(Size);
  Result := StrAlloc(Size);
  SendMessage(Memo.Handle, WM_GETTEXT, Size, Longint(Result));
end;

procedure SetMemoText(Memo: TCustomEdit; Buf: PChar);
begin
  SendMessage(Memo.Handle, WM_SETTEXT, 0, Longint(Buf));
end;

procedure Res2Memo(Memo: TCustomEdit; ID: Word; Res: PChar);
var
  Buffer: PChar;
begin
  Buffer := nil;
  if (ID = 0) and (Res = nil) then Exit; {wrong input. do nothing}
  try
    if ID = 0 then Buffer := GetResBuffer(Res)
    else
    begin
      Buffer := GetClearStr(256);
      StrPCopy(Buffer, LoadStr(ID));
    end;
    SetMemoText(Memo, Buffer);
  finally
    if Assigned(Buffer) then StrDispose(Buffer);
  end;
end;

procedure Buf2Memo_Free(Memo: TCustomEdit; var Buf: PChar);
begin
  try
    SendMessage(Memo.Handle, WM_SETTEXT, 0, Longint(Buf));
  finally
    StrDispose(Buf);
    Buf := nil;
  end;
end;

function Memo2BuffIsOk(Memo: TCustomEdit; var Buff: PChar): Boolean;
begin
  Buff := GetMemo2Buffer(Memo);
  Result := Buff <> nil;
end;

function GetMemo2BufferSize(Memo: TCustomEdit; Size: Cardinal): PChar;
var
  Len: Cardinal;
begin
  Result := nil;
  Len := MemoGetTextLen(Memo);
  if Len = 0 then Exit;
  if (Size = 0) or (Size > Len) then Size := Len;
  Inc(Size);
  Result := StrAlloc(Size);
  SendMessage(Memo.Handle, WM_GETTEXT, Size, Longint(Result));
end;

function GetLimitedMemo(Memo: TCustomEdit; Size: Cardinal; Buff: PChar): PChar;
var
  Len: Cardinal;
begin
  Result := nil;
  Len := MemoGetTextLen(Memo);
  if Len = 0 then Exit;
  if (Size = 0) or (Size > Len) then Size := Len;
  Inc(Size);
  SendMessage(Memo.Handle, WM_GETTEXT, Size, Longint(Buff));
  Result := Buff;
end;

procedure ConcatIniSections(IniFile: TIniFile; var Strings: TStringList;
  Sections: array of String);
var
  I, K: Integer;
  S: TStringList;
begin
  S := TStringList.Create;
  try
    for I := Low(Sections) to High(Sections) do
    begin
      S.Clear;
      IniFile.ReadSectionValues(Sections[I], S);
      for K := 0 to S.Count - 1 do
	Strings.Add(S[K]);
    end;
  finally
    S.Free;
  end;
end;

function WriteMemoToFile(Memo: TCustomEdit; FileName: String): Boolean;
var
  Buffer: PChar;
begin
  Buffer := GetMemo2Buffer(Memo);
  try
    Result := WriteFileFromBuff(FileName, Buffer, False);
  finally
    StrDispose(Buffer);
  end;
end;

procedure File2Memo(FileName: String; Memo: TCustomEdit);
var
  Buffer: PChar;
begin
  Buffer := ReadFileToBuffer(FileName);
  if Assigned(Buffer) then
  try
    UpdateMemoFromBuffer(Memo, Buffer);
  finally
    StrDispose(Buffer);
  end;
end;

type TLocalControl = class(TControl) end;

procedure ResetCtlsColor(Sender : TForm; OldActive, NewActive: TWinControl;
  clActiveField, clInactiveField, clFontColor: TColor);

procedure SetColor(C: TComponent; Active: Boolean);
var
  Color: TColor;
begin
  if not Assigned(C) then Exit;
  if Active then Color := clActiveField
  else Color := clInactiveField;
  TLocalControl(C).Color := Color;
  TLocalControl(C).Font.Color := clFontColor;
  TLocalControl(C).Invalidate;
end;

var
  I: Integer;
  COld, CNew: TComponent;
begin
  COld := nil;
  CNew := nil;
  with Sender do
    for I := 0 to ComponentCount - 1 do
    if not (Components[I].InheritsFrom(TScrollBox) or
            Components[I].InheritsFrom(TNoteBook)) then
    begin
      if (Components[I] = OldActive) then COld := Components[I];
      if (Components[I] = NewActive) then CNew := Components[I];
    end;
  SetColor(COld, False);
  SetColor(CNew, True);
end;

procedure Resource2Memo(Memo: TCustomEdit; ResName: PChar);
var
  Buffer: PChar;
begin
  Buffer := GetResBuffer(ResName);
  if Assigned(Buffer) then
  try
    UpdateMemoFromBuffer(Memo, Buffer);
  finally
    StrDispose(Buffer);
  end;
end;

procedure PaintFormBackg(Form: TForm; Bounds: TRect);
var
  HB, HG, I, Height, Width: Integer;
  Brush: HBrush;
begin
  with Bounds do
  begin
    Width := Right - Left;
    Height := Bottom - Top;
  end;
  for I := 0 to Height do
  begin
    HG := Trunc(155.0/Height * I);
    HB := Trunc(255.0/Height * I);
    Brush := CreateSolidBrush(RGB(80, 155 - HG, 255 - HB));
    try
      FillRect(Form.Canvas.Handle,
               Rect(Bounds.Left, Bounds.Top + I, Width, Bounds.Top + I + 1), Brush);
    finally
      DeleteObject(Brush);
    end;
  end;
end;

procedure CenterFormOverApp(F: TForm);
var
  X, Y: Integer;
  FOwner: TForm;
begin
  if (Application.MainForm = nil) or not Application.MainForm.Visible then Exit;
  if IsWindowVisible(F.Handle) and (F.Tag <> 2000) then Exit; {as per 32-experts}
  if (F.Owner = Application) then FOwner := TForm(Application.MainForm)
  else
  begin
    FOwner := TForm(F.Owner);
    if (FOwner.FormStyle = fsMDIChild) or (FOwner.BorderStyle = bsDialog) then
      FOwner := TForm(Application.MainForm);
  end;

  with FOwner do
  begin
    X := Left + (Width  - F.Width)  div 2;
    Y := Top  + (Height - F.Height) div 2;
    if X < 0 then X := Left + 20;
    if Y < 0 then Y := Top + 50;
  end;
  F.SetBounds(X, Y, F.Width, F.Height);
end;

procedure CenterWndOverApp(Wnd: HWND);
var
  X, Y: Integer;
  R: TRect;
begin
  if Application.MainForm = nil then Exit;
  if not Application.MainForm.Visible then Exit;
  GetWindowRect(Wnd, R);
  with Application.MainForm do
  begin
    X := Left + (Width - (R.Right - R.Left)) div 2;
    Y := Top  + (Height - (R.Bottom - R.Top)) div 2;
    if X < 0 then X := Left + 20;
    if Y < 0 then Y := Top + 50;
  end;
  SetWindowPos(Wnd, HWND_TOP, X, Y, 0, 0, SWP_NOZORDER or SWP_NOSIZE);
end;

procedure Make_BMP_Transparent(I: TImage; BckgColor: TColor);
var
  R: TRect;
  TempBMP: TBitmap;
begin
  TempBMP := TBitMap.Create;
  try
    TempBMP.Assign(I.Picture.BitMap);
    with TempBMP do R := Rect(0, 0, Width, Height);
    TempBMP.Canvas.Brush.Color := BckgColor;
    TempBMP.Canvas.BrushCopy(R, TempBMP, R, clWhite);
    I.Picture.BitMap.Assign(TempBMP);
  finally
    TempBMP.Free;
  end;
end;

procedure Initialize_ListBox(LB: TListBox);
var
  OldFont: HFONT;
  TM: TTextMetric;
begin
  OldFont := SelectObject(LB.Canvas.Handle, LB.Font.Handle);  { select our font }
  GetTextMetrics(LB.Canvas.Handle, TM);                       { retrieves font metrics }
  LB.ItemHeight := TM.tmHeight+TM.tmInternalLeading;          { measure height of text }
  SelectObject(LB.Canvas.Handle, OldFont);                    { select back system font }
  LB.Canvas.Pen.Color := clGrayText;                          { set pen color (grids) }
end;

{ To ensure correct bitmap zooming we reload the original bitmap with a }
{ call to GetSpeedBtnGlyph, than we loop to required zoom using a 1.25  }
{ ratio as per using ScaleBy(5, 4) to zoom forms                        }
procedure ZoomGlyph(B: TComponent; OldZoom, NewZoom: Byte;
  GetSpeedBtnGlyph: TGetOriginalGlyphFunc);
var
  DstRect: TRect;
  I: Byte;
  W: Extended;
  ResizedBmp: TBitmap;
  Glyph: TBitmap;
begin
  if NewZoom = OldZoom then Exit;                          {nothing to do}
  if B.InheritsFrom(TBitBtn) then Glyph := TBitBtn(B).Glyph           {map glyph}
  else
    if B.InheritsFrom(TSpeedButton) then Glyph := TSpeedButton(B).Glyph
    else Exit;
  if not GetSpeedBtnGlyph(B, Glyph) then Exit;             {load unzoomed glyph or exit}
  if NewZoom = 0 then Exit;                                {nothing to do}
  W := Glyph.Height;                                       {initialize}
  for I := 0 to Pred(NewZoom) do W := W * 1.25;            {correspond to scaleby(5,4)}
  if B.InheritsFrom(TBitBtn) then
    with TBitBtn(B) do
      DstRect := Rect(0, 0, Trunc(W) * NumGlyphs, Trunc(W)){calc dest rectangle}
  else
    with TSpeedButton(B) do
      DstRect := Rect(0, 0, Trunc(W) * NumGlyphs, Trunc(W));{calc dest rectangle}
  ResizedBmp := TBitmap.Create;                            {allocate tbitmap}
  try
    ResizedBmp.Width := DstRect.Right;                     {set width of temp bitmap}
    ResizedBmp.Height := DstRect.Bottom;                   {set height of temp bitmap}
    ResizedBmp.Canvas.StretchDraw(DstRect, Glyph);         {stretch glyph}
    Glyph.Assign(ResizedBmp);                              {GRAB IT}
  finally
    ResizedBmp.Free;                                       {free temp bitmap}
  end;
end;

initialization
  CenterDialogsOverApp := False;
end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

